@@ -1,5 +1,5 @@
 /*
- * Copyright 2010 - 2012 Michael Ossmann
+ * Copyright 2010-2017 Great Scott Gadgets <info@greatscottgadgets.com>
  *
  * This file is part of HackRF.
  *
@@ -20,13 +20,20 @@
  */
 
 #include "hackrf_core.h"
+#include "platform_detect.h"
 
 int main(void)
 {
+	detect_hardware_platform();
 	pin_setup();
 
-	/* enable all power supplies */
+#ifndef PRALINE
+	/* enable 1V8 power supply so that the 1V8 LED lights up */
 	enable_1v8_power();
+#else
+	/* enable 1V2 power supply so that the 3V3FPGA LED lights up */
+	enable_1v2_power();
+#endif
 
 	/* Blink LED1/2/3 on the board. */
 	while (1) 
