@@ -1,5 +1,5 @@
 /*
- * Copyright 2012 Michael Ossmann
+ * Copyright 2012-2014 Great Scott Gadgets <info@greatscottgadgets.com>
  * Copyright 2014 Jared Boone <jared@sharebrained.com>
  *
  * This file is part of HackRF.
@@ -34,6 +34,9 @@
 typedef struct {
 	spi_bus_t* const bus;
 	gpio_t gpio_reset;
+#ifdef PRALINE
+	gpio_t gpio_ld;
+#endif
 	uint16_t regs[RFFC5071_NUM_REGS];
 	uint32_t regs_dirty;
 } rffc5071_driver_t;
@@ -55,8 +58,8 @@ extern void rffc5071_reg_write(rffc5071_driver_t* const drv, uint8_t r, uint16_t
  * provided routines for those operations. */
 extern void rffc5071_regs_commit(rffc5071_driver_t* const drv);
 
-/* Set frequency (MHz). */
-extern uint64_t rffc5071_set_frequency(rffc5071_driver_t* const drv, uint16_t mhz);
+/* Set frequency (Hz). */
+extern uint64_t rffc5071_set_frequency(rffc5071_driver_t* const drv, uint64_t hz);
 
 /* Set up rx only, tx only, or full duplex. Chip should be disabled
  * before _tx, _rx, or _rxtx are called. */
@@ -67,5 +70,8 @@ extern void rffc5071_enable(rffc5071_driver_t* const drv);
 extern void rffc5071_disable(rffc5071_driver_t* const drv);
 
 extern void rffc5071_set_gpo(rffc5071_driver_t* const drv, uint8_t);
+#ifdef PRALINE
+extern bool rffc5071_poll_ld(rffc5071_driver_t* const drv, uint8_t* prelock_state);
+#endif
 
 #endif // __RFFC5071_H
