@@ -1,4 +1,5 @@
 /*
+ * Copyright 2012-2022 Great Scott Gadgets <info@greatscottgadgets.com>
  * Copyright 2012 Jared Boone <jared@sharebrained.com>
  *
  * This file is part of HackRF.
@@ -35,26 +36,21 @@ typedef enum {
 } sgpio_direction_t;
 
 typedef struct sgpio_config_t {
-	gpio_t gpio_rx_q_invert;
-	gpio_t gpio_hw_sync_enable;
+	gpio_t gpio_q_invert;
+#ifndef PRALINE
+	gpio_t gpio_trigger_enable;
+#endif
 	bool slice_mode_multislice;
 } sgpio_config_t;
 
 void sgpio_configure_pin_functions(sgpio_config_t* const config);
 void sgpio_test_interface(sgpio_config_t* const config);
-void sgpio_set_slice_mode(
-	sgpio_config_t* const config,
-	const bool multi_slice
-);
-void sgpio_configure(
-	sgpio_config_t* const config,
-	const sgpio_direction_t direction
-);
+void sgpio_set_slice_mode(sgpio_config_t* const config, const bool multi_slice);
+void sgpio_configure(sgpio_config_t* const config, const sgpio_direction_t direction);
 void sgpio_cpld_stream_enable(sgpio_config_t* const config);
 void sgpio_cpld_stream_disable(sgpio_config_t* const config);
 bool sgpio_cpld_stream_is_enabled(sgpio_config_t* const config);
 
-bool sgpio_cpld_stream_rx_set_decimation(sgpio_config_t* const config, const uint_fast8_t n);
-void sgpio_cpld_stream_rx_set_q_invert(sgpio_config_t* const config, const uint_fast8_t invert);
+void sgpio_cpld_set_mixer_invert(sgpio_config_t* const config, uint_fast8_t invert);
 
-#endif//__SGPIO_H__
+#endif //__SGPIO_H__
